package pen;

import java.io.StringReader;

import pen_flowchart.Parts;
import pen_flowchart.Parts_Bar;
import pen_flowchart.Parts_LoopBegin;
import pen_flowchart.Parts_LoopBegin1;
import pen_flowchart.Parts_LoopBegin2;
import pen_flowchart.Parts_LoopBeginDec;
import pen_flowchart.Parts_LoopBeginInc;
import pen_flowchart.Parts_LoopEnd2;
import pen_flowchart.Parts_LoopEndEmpty;
import pen_flowchart.Parts_Misc;
import pen_flowchart.Parts_Null;
import pen_flowchart.Parts_Input;
import pen_flowchart.Parts_Output;
import pen_flowchart.Parts_Substitute;
import pen_flowchart.Parts_If;

public class Pen2Flowchart implements IntVParserTreeConstants{
	static MainGUI gui;
	static void setMainGUI(MainGUI gui)
	{
		Pen2Flowchart.gui = gui;
	}
	static public boolean convert(String s, pen_flowchart.Flowchart chart, String[] vars)
	{
		IntVParser parser = new IntVParser(new StringReader(s + "\n"));
		parser.setMainGUI(gui);
		parser.disable_tracing();
		try {
			parser.IntVUnit();
			Node root = parser.jjtree.rootNode();
			pen_flowchart.Parts parts = chart.getFirstParts();
			return convertPen2Flowchart(root, parts, vars);
		} catch (ParseException e) {
			return false;
		} catch (TokenMgrError e) {
			return false;
		} catch (Exception e) {
			return false;
		}
	}
	static private boolean convertPen2Flowchart(Node node, pen_flowchart.Parts parts, String[] vars) throws ParseException
	{
		for(int i = 0; i < node.jjtGetNumChildren(); i++)
		{
			SimpleNode child = (SimpleNode) node.jjtGetChild(i);
			switch(child.id)
			{
			case JJTVARDECL: // 変数宣言
				setVars((ASTVarDecl)child, vars);
				break;
			case JJTASSIGNSTATS: // 代入
				parts = makeSubstitute((ASTAssignStats) child, parts);
				break;
			case JJTASSIGNSTAT: // 入力
				parts = makeGet((ASTAssignStat)child, parts);
				break;
			case JJTASSIGNARRAYSTAT:
				parts = makeSubstituteArray((ASTAssignArrayStat) child, parts);
				break;
			case JJTPUTSTAT: // 出力
				parts = makePut((ASTPutStat)child, parts);
				break;
			case JJTIFSTAT: // もし
				parts = makeIf((ASTIfStat) child, parts);
				break;
			case JJTWHILESWITCHFOR: // 〜の間くりかえす or 増やしながら(減らしながら)繰り返す
				parts = makeWhileswitchfor((ASTWhileSwitchFor)child, parts);
				break;
			case JJTREPEATUNTIL:
				parts = makeRepeatuntil((ASTRepeatUntil)child, parts);
				break;
			case JJTGOPENWINDOW:
				parts = makeMisc("gOpenWindow", child.children, parts);	break;
			case JJTGOPENGRAPHWINDOW:
				parts = makeMisc("gOpenGraphWindow", child.children, parts);	break;
			case JJTGCLOSEWINDOW:
				parts = makeMisc("gCloseWindow", child.children, parts);	break;
			case JJTGCLEARWINDOW:
				parts = makeMisc("gClearWindow", child.children, parts);	break;
			case JJTGSAVEWINDOW:
				parts = makeMisc("gSaveWindow", child.children, parts);	break;
			case JJTGSETORIGIN:
				parts = makeMisc("gSetOrigin", child.children, parts);	break;
			case JJTGSETMAP:
				parts = makeMisc("gSetMap", child.children, parts);	break;
			case JJTGSETDOTSHAPE:
				parts = makeMisc("gSetDotShape", child.children, parts);	break;
			case JJTGSETLINECOLOR:
				parts = makeMisc("gSetLineColor", child.children, parts);	break;
			case JJTGSETFILLCOLOR:
				parts = makeMisc("gSetFillColor", child.children, parts);	break;
			case JJTGSETLINESHAPE:
				parts = makeMisc("gSetLineShape", child.children, parts);	break;
			case JJTGSETLINEWIDTH:
				parts = makeMisc("gSetLineWidth", child.children, parts);	break;
			case JJTGSETARROWTYPE:
				parts = makeMisc("gSetArrowType", child.children, parts);	break;
			case JJTGSETARROWDIR:
				parts = makeMisc("gSetArrowDir", child.children, parts);	break;
			case JJTGSETTEXTCOLOR:
				parts = makeMisc("gSetTextColor", child.children, parts);	break;
			case JJTGSETFONT:
				parts = makeMisc("gSetFont", child.children, parts);	break;
			case JJTGSETFONTTYPE:
				parts = makeMisc("gSetFontType", child.children, parts);	break;
			case JJTGSETFONTSIZE:
				parts = makeMisc("gSetFontSize", child.children, parts);	break;
			case JJTGDRAWPOINT:
				parts = makeMisc("gDrawPoint", child.children, parts);	break;
			case JJTGDRAWLINE:
				parts = makeMisc("gDrawLine", child.children, parts);	break;
			case JJTGDRAWTEXT:
				parts = makeMisc("gDrawText", child.children, parts); break;
			case JJTGDRAWCIRCLE:
				parts = makeMisc("gDrawCircle", child.children, parts);	break;
			case JJTGFILLCIRCLE:
				parts = makeMisc("gFillCircle", child.children, parts);	break;
			case JJTGDRAWOVAL:
				parts = makeMisc("gDrawOval", child.children, parts);	break;
			case JJTGFILLOVAL:
				parts = makeMisc("gFillOval", child.children, parts);	break;
			case JJTGDRAWBOX:
				parts = makeMisc("gDrawBox", child.children, parts);	break;
			case JJTGFILLBOX:
				parts = makeMisc("gFillBox", child.children, parts);	break;
			case JJTGDRAWARC:
				parts = makeMisc("gDrawArc", child.children, parts);	break;
			case JJTGFILLARC:
				parts = makeMisc("gFillArc", child.children, parts);	break;
			case JJTGDRAWPOLYGON:
				parts = makeMisc("gDrawPolygon", child.children, parts);	break;
			case JJTGFILLPOLYGON:
				parts = makeMisc("gFillPolygon", child.children, parts);	break;
			case JJTGDRAWPOLYLINE:
				parts = makeMisc("gDrawPolyline", child.children, parts);	break;
			case JJTGDRAWIMAGE:
				parts = makeMisc("gDrawImage", child.children, parts);	break;
			case JJTBREAK:
				parts = makeMisc("break", child.children, parts);	break;
			case JJTSLEEP:
				parts = makeMisc("sleep", child.children, parts);	break;
			case JJTFILE_CLOSE:
				parts = makeMisc("close", child.children, parts);	break;
			case JJTFILE_PUTSTR:
				parts = makeMisc("putstr", child.children, parts);	break;
			case JJTFILE_PUTLINE:
				parts = makeMisc("putline", child.children, parts);	break;
			case JJTFILE_RENAME:
				parts = makeMisc("rename", child.children, parts);	break;
			case JJTFILE_FLUSH:
				parts = makeMisc("flush", child.children, parts);	break;
			case JJTFILE_REMOVE:
				parts = makeMisc("remove", child.children, parts);	break;
			case JJTFUNCTIONCALL:
				parts = makeFunctionCall((ASTFunctionCall)child, parts); break;
			default:	// 手に負えない
				return false;
			}
		}
		return true;
	}


	static private void setVars(ASTVarDecl child, String[] vars) throws ParseException
	{
		if(child.decl >= 1 && child.decl <= 4)
		{
			StringBuilder s = new StringBuilder(vars[child.decl - 1]);
			for(int i = 0; i < child.jjtGetNumChildren(); i++)
			{
				if(s.length() > 0) s.append(',');
				setValue(child.jjtGetChild(i), s);
			}
			vars[child.decl - 1] = s.toString();
		}
	}

	static private void setValue(Node node, StringBuilder s) throws ParseException
	{
		switch(((SimpleNode)node).id)
		{
		case JJTDECL:
			s.append(((ASTDecl)node).varName);
			appendChild(node, s, null, null, ",");
			break;
		case JJTIDENT:	// 関数呼び出しはここで処理する
			s.append(((ASTIdent)node).varName);
			appendChild(node, s, null, null, ",");
			break;
		case JJTARRAY:
			s.append(((SimpleNode)((SimpleNode)node).parent).id == JJTARRAY ? ',' :  '[');
			appendChild(node, s, null, null, null);
			if(((SimpleNode)((SimpleNode)node).parent).id != JJTARRAY)s.append(']');
			break;
		case JJTARRAYNUM:
			s.append(((SimpleNode)((SimpleNode)node).parent).id == JJTARRAYNUM ? ',' :  '[');
			appendChild(node, s, null, null, null);
			if(((SimpleNode)((SimpleNode)node).parent).id != JJTARRAYNUM)s.append(']');
			break;
		case JJTSTRLIT:
			s.append("「");
			{
				String temp = ((ASTStrlit)node).litString;
				s.append(temp.replaceAll("\\n", "\\\\n").replaceAll("\\r", "\\\\r").replaceAll("\\t", "\\\\t")
						.replaceAll("「", "\\\\「").replaceAll("」", "\\\\」").replaceAll("\\\"","\\\\\""));
			}
			s.append("」");
			break;
		case JJTFLOATLITERAL:
			double v = (Double)((ASTFloatLiteral)node).litValue;
			String sv = String.valueOf(v);
			if(sv.indexOf('E') >= 0) s.append(String.format("%f", v));
			else s.append(sv);
			break;
		case JJTLITERAL:
			s.append(((ASTLiteral)node).litValue.toString());
			break;
		case JJTMINNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTADDNODE: case JJTSUBNODE: case JJTMULNODE: case JJTDIVNODE: case JJTSURNODE: case JJTMINNODE:
				s.append('('); break;
			}
			s.append('-');
			setValue(((SimpleNode)node).jjtGetChild(0), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTADDNODE: case JJTSUBNODE: case JJTMULNODE: case JJTDIVNODE: case JJTSURNODE: case JJTMINNODE:
				s.append(')'); break;
			}
			break;
		case JJTADDNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMULNODE: case JJTDIVNODE: case JJTSURNODE: case JJTMINNODE:
				s.append('('); break;
			case JJTSUBNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append('(');
			}
			setValue(((SimpleNode)node).jjtGetChild(0), s);
			s.append('+');
			setValue(((SimpleNode)node).jjtGetChild(1), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMULNODE: case JJTDIVNODE: case JJTSURNODE: case JJTMINNODE:
				s.append(')'); break;
			case JJTSUBNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append(')');
			}
			break;
		case JJTSUBNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMULNODE: case JJTDIVNODE: case JJTSURNODE: case JJTMINNODE:
				s.append('('); break;
			case JJTSUBNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append('(');
			}
			setValue(((SimpleNode)node).jjtGetChild(0), s);
			s.append('-');
			setValue(((SimpleNode)node).jjtGetChild(1), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMULNODE: case JJTDIVNODE: case JJTSURNODE: case JJTMINNODE:
				s.append(')'); break;
			case JJTSUBNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append(')');
				break;
			}
			break;
		case JJTMULNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMINNODE:
				s.append('('); break;
			case JJTDIVNODE: case JJTMULNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)s.append('('); break;
			}
			setValue(((SimpleNode)node).jjtGetChild(0), s);
			s.append('*');
			setValue(((SimpleNode)node).jjtGetChild(1), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMINNODE:
				s.append(')'); break;
			case JJTDIVNODE: case JJTMULNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)s.append(')'); break;
			}
			break;
		case JJTDIVNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMINNODE:
				s.append('('); break;
			case JJTDIVNODE: case JJTSURNODE: case JJTMULNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append('(');
				break;
			}
			setValue(((SimpleNode)node).jjtGetChild(0), s);
			s.append('/');
			setValue(((SimpleNode)node).jjtGetChild(1), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMINNODE:
				s.append(')'); break;
			case JJTDIVNODE: case JJTSURNODE: case JJTMULNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append(')');
				break;
			}
			break;
		case JJTSURNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMINNODE:
				s.append('('); break;
			case JJTDIVNODE: case JJTSURNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append('(');
				break;
			}
			setValue(((SimpleNode)node).jjtGetChild(0), s);
			s.append('%');
			setValue(((SimpleNode)node).jjtGetChild(1), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTMINNODE:
				s.append(')'); break;
			case JJTDIVNODE: case JJTSURNODE:
				if(((SimpleNode)node).parent.jjtGetChild(1) == node)
					s.append(')');
				break;
			}
			break;
		case JJTPUTSTAT:		appendChild(node, s, null, null, "と"); break;
		case JJTRANDOM:		appendChild(node, s, "random(", ")", ","); break;
		case JJTSINE:			appendChild(node, s, "sin(", ")", ","); break;
		case JJTCOSINE:		appendChild(node, s, "cos(", ")", ","); break;
		case JJTTANGENT:		appendChild(node, s, "tan(", ")", ","); break;
		case JJTSQRT:			appendChild(node, s, "sqrt(", ")", ","); break;
		case JJTFLOOR:		appendChild(node, s, "floor(", ")", ","); break;
		case JJTCEIL:			appendChild(node, s, "ceil(", ")", ","); break;
		case JJTROUND:		appendChild(node, s, "round(", ")", ","); break;
		case JJTABS:			appendChild(node, s, "abs(", ")", ","); break;
		case JJTLOG:			appendChild(node, s, "log(", ")", ","); break;
		case JJTINT:			appendChild(node, s, "int(", ")", ","); break;
		case JJTLENGTH:		appendChild(node, s, "length(", ")", ","); break;
		case JJTAPPEND:		appendChild(node, s, "append(", ")", ","); break;
		case JJTSUBSTRING:	appendChild(node, s, "substring(", ")", ","); break;
		case JJTINSERT:		appendChild(node, s, "insert(", ")", ","); break;
		case JJTREPLACE:		appendChild(node, s, "replace(", ")", ","); break;
		case JJTEXTRACT:		appendChild(node, s, "extract(", ")", ","); break;
		case JJTSTR2INT:		appendChild(node, s, "str2int(", ")", ","); break;
		case JJTINT2STR:		appendChild(node, s, "int2str(", ")", ","); break;
		case JJTCOMPARE:		appendChild(node, s, "compare(", ")", ","); break;
		case JJTFILE_OPENR:	appendChild(node, s, "openr(", ")", ","); break;
		case JJTFILE_OPENW:	appendChild(node, s, "openw(", ")", ","); break;
		case JJTFILE_OPENA:	appendChild(node, s, "opena(", ")", ","); break;
		case JJTFILE_GETSTR:	appendChild(node, s, "getstr(", ")", ","); break;
		case JJTFILE_GETLINE:appendChild(node, s, "getline(", ")", ","); break;
		case JJTFILE_ISFILE:	appendChild(node, s, "isfile(", ")", ","); break;
		case JJTGET:			s.append("input()"); break;
		case JJTEQNODE: case JJTNTNODE: case JJTLSNODE: case JJTGTNODE: case JJTLENODE: case JJTGENODE:
			if(((SimpleNode)((SimpleNode)node).parent).id == JJTASSIGNSTAT) s.append('(');
			switch(((SimpleNode)node).id)
			{
			case JJTEQNODE:
				setValue(node.jjtGetChild(0), s); s.append('='); setValue(node.jjtGetChild(1), s); break;
			case JJTNTNODE:
				setValue(node.jjtGetChild(0), s); s.append("!="); setValue(node.jjtGetChild(1), s); break;
			case JJTLSNODE:
				setValue(node.jjtGetChild(0), s); s.append('<'); setValue(node.jjtGetChild(1), s); break;
			case JJTGTNODE:
				setValue(node.jjtGetChild(0), s); s.append('>'); setValue(node.jjtGetChild(1), s); break;
			case JJTLENODE:
				setValue(node.jjtGetChild(0), s); s.append("<="); setValue(node.jjtGetChild(1), s); break;
			case JJTGENODE:
				setValue(node.jjtGetChild(0), s); s.append(">="); setValue(node.jjtGetChild(1), s); break;
			}
			if(((SimpleNode)((SimpleNode)node).parent).id == JJTASSIGNSTAT) s.append(')');
			break;
		case JJTORNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTANDNODE: case JJTNOTNODE: case JJTASSIGNSTAT:
				s.append('('); break;
			}
			setValue(node.jjtGetChild(0), s); s.append(" または "); setValue(node.jjtGetChild(1), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTANDNODE: case JJTNOTNODE: case JJTASSIGNSTAT:
				s.append(')'); break;
			}
			break;
		case JJTANDNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTORNODE: case JJTNOTNODE: case JJTASSIGNSTAT:
				s.append('('); break;
			}
			setValue(node.jjtGetChild(0), s); s.append(" かつ "); setValue(node.jjtGetChild(1), s);
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTORNODE: case JJTNOTNODE: case JJTASSIGNSTAT:
				s.append(')'); break;
			}
			break;
		case JJTNOTNODE:
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTORNODE: case JJTNOTNODE: case JJTASSIGNSTAT:
				s.append('('); break;
			}
			setValue(node.jjtGetChild(0), s); s.append(" でない");
			switch(((SimpleNode)((SimpleNode)node).parent).id)
			{
			case JJTORNODE: case JJTNOTNODE: case JJTASSIGNSTAT:
				s.append(')'); break;
			}
			break;
		case JJTEXTRA_STR:
			switch(((ASTEXTRA_STR)node).char_code[0])
			{
			case 0xFFFF: s.append("EOF"); break;
			case 0x0000: s.append("NULL"); break;
			case 0x000D: s.append("CR"); break;
			case 0x000A: s.append("LF"); break;
			case 0x0009: s.append("HT"); break;
			}
			break;
		case JJTTRUE:		s.append("TRUE"); break;
		case JJTFALSE:	s.append("FALSE"); break;
		default:

			throw new ParseException(node.toString());
		}
	}

	static private void appendChild(Node node, StringBuilder s, String header, String footer, String connector) throws ParseException
	{
		if(header != null && node.jjtGetNumChildren() > 0) s.append(header);
		for(int i = 0; i < node.jjtGetNumChildren(); i++)
		{
			if(i > 0 && connector != null) s.append(connector);
			setValue((SimpleNode)node.jjtGetChild(i), s);
		}
		if(footer != null && node.jjtGetNumChildren() > 0) s.append(footer);
	}

	static private Parts makeSubstitute(ASTAssignStats node, Parts parts0) throws ParseException {
		Parts parts;
		for(int i = 0; i < node.jjtGetNumChildren(); i++)
		{
			ASTAssignStat stat = (ASTAssignStat)node.jjtGetChild(i);

			if(stat.jjtGetChild(1) instanceof ASTGetStat)
			{
				StringBuilder var = new StringBuilder();
				setValue(stat.jjtGetChild(0), var);
				parts = new Parts_Input();
				((Parts_Input)parts).setValue(var.toString());
			}
			else
			{
				StringBuilder var = new StringBuilder();
				setValue(stat.jjtGetChild(0), var);
				switch(((SimpleNode)stat.jjtGetChild(1)).id)
				{
				case JJTFILE_OPENR:
					parts = makeMiscSubstitute("openr", var.toString(), (SimpleNode)stat.jjtGetChild(1)); break;
				case JJTFILE_OPENW:
					parts = makeMiscSubstitute("openw", var.toString(), (SimpleNode)stat.jjtGetChild(1)); break;
				case JJTFILE_OPENA:
					parts = makeMiscSubstitute("opena", var.toString(), (SimpleNode)stat.jjtGetChild(1)); break;
				case JJTFILE_GETSTR:
					parts = makeMiscSubstitute("getstr", var.toString(), (SimpleNode)stat.jjtGetChild(1)); break;
				case JJTFILE_GETLINE:
					parts = makeMiscSubstitute("getline", var.toString(), (SimpleNode)stat.jjtGetChild(1)); break;
				case JJTFILE_ISFILE:
					parts = makeMiscSubstitute("isfile", var.toString(), (SimpleNode)stat.jjtGetChild(1)); break;
				default:
					StringBuilder val = new StringBuilder();
					setValue(stat.jjtGetChild(1), val);
					parts = new Parts_Substitute();
					((Parts_Substitute)parts).setValue(var.toString(), val.toString());
				}
			}
			parts0.connectLeft(parts);
			parts0 = new Parts_Bar();
			parts.connectLeft(parts0);
		}
		return parts0;
	}
	private static Parts makeSubstituteArray(ASTAssignArrayStat child, Parts parts0) throws ParseException {
		Parts parts;
		StringBuilder var = new StringBuilder();
		setValue(child.jjtGetChild(0), var);
		StringBuilder val = new StringBuilder("{");
		for(int i = 1; i < child.jjtGetNumChildren(); i++)
		{
			if(i > 1) val.append(',');
			setValue(child.jjtGetChild(i),val);
		}
		val.append("}");
		parts = new Parts_Substitute();
		((Parts_Substitute)parts).setValue(var.toString(), val.toString());
		parts0.connectLeft(parts);
		parts0 = new Parts_Bar();
		parts.connectLeft(parts0);

		return parts0;
	}

	static private Parts makeGet(ASTAssignStat node, Parts parts0) throws ParseException {
		Parts_Input parts = new Parts_Input();
		StringBuilder var = new StringBuilder();
		setValue(node.jjtGetChild(0), var);
		parts.setValue(var.toString());
		parts0.connectLeft(parts);
		Parts_Bar bar = new Parts_Bar();
		parts.connectLeft(bar);
		return bar;
	}

	static private Parts makePut(ASTPutStat node, Parts parts0) throws ParseException {
		Parts_Output parts = new Parts_Output();
		StringBuilder val = new StringBuilder();
		setValue(node, val);
		parts.setValue(val.toString(), node.n == "");
		parts0.connectLeft(parts);
		Parts_Bar bar = new Parts_Bar();
		parts.connectLeft(bar);
		return bar;
	}

	static private Parts makeIf(ASTIfStat node, Parts parts0) throws ParseException {
		Parts_If parts = new Parts_If();
		Parts_Bar bar1 = new Parts_Bar();
		Parts_Bar bar2 = new Parts_Bar();
		Parts_Bar bar3 = new Parts_Bar();
		Parts_Null pend = new Parts_Null();

		parts.setEndif(pend);
		parts0.connectLeft(parts);
		parts.connectLeft(bar1);
		bar1.connectLeft(pend);
		pend.connectLeft(bar3);
		parts.setRight(bar2);
		bar2.setLeft(pend);

		StringBuilder cond = new StringBuilder();
		setValue(node.jjtGetChild(0), cond);
		parts.setValue(cond.toString());

		convertPen2Flowchart(node.jjtGetChild(1), bar1, null);
		if(node.jjtGetNumChildren() > 2)
		{
			SimpleNode elsenode = (SimpleNode)node.jjtGetChild(2);
			if(elsenode.id == JJTIFSTAT)
				makeIf((ASTIfStat) elsenode, bar2);
			else
				convertPen2Flowchart(node.jjtGetChild(2), bar2, null);
		}
		return bar3;
	}

	static private Parts makeWhileswitchfor(ASTWhileSwitchFor node, Parts parts) throws ParseException {
		if(node.jjtGetNumChildren() == 2)
		{
			Parts_LoopBegin1 p1 = new Parts_LoopBegin1();
			Parts_LoopEndEmpty p2 = new Parts_LoopEndEmpty();
			p1.setLoopEnd(p2);
			p2.setLoopBegin(p1);
			Parts_Bar b1 = new Parts_Bar();
			Parts_Bar b2 = new Parts_Bar();
			parts.connectLeft(p1);
			p1.connectLeft(b1);
			b1.connectLeft(p2);
			p2.connectLeft(b2);
			StringBuilder cond = new StringBuilder();
			setValue(node.jjtGetChild(0), cond);
			p1.setValue(cond.toString());
			convertPen2Flowchart(node.jjtGetChild(1), b1, null);
			return b2;
		}
		else
		{
			Parts_LoopBegin p1;
			Parts_LoopEndEmpty p2 = new Parts_LoopEndEmpty();
			int direction = ((ASTForStatAdd)node.jjtGetChild(3)).op;
			p1 = direction > 0 ? new Parts_LoopBeginInc() : new Parts_LoopBeginDec();
			p1.setLoopEnd(p2);
			p2.setLoopBegin(p1);
			Parts_Bar b1 = new Parts_Bar();
			Parts_Bar b2 = new Parts_Bar();
			parts.connectLeft(p1);
			p1.connectLeft(b1);
			b1.connectLeft(p2);
			p2.connectLeft(b2);
			StringBuilder var = new StringBuilder();
			StringBuilder val1 = new StringBuilder();
			StringBuilder val2 = new StringBuilder();
			StringBuilder val3 = new StringBuilder();
			setValue(node.jjtGetChild(0), var);
			setValue(node.jjtGetChild(1), val1);
			setValue(node.jjtGetChild(2), val2);
			if(node.jjtGetChild(3).jjtGetNumChildren() > 0)
				setValue(node.jjtGetChild(3).jjtGetChild(0), val3);
			else
				val3.append("1");
			if(direction > 0)
				((Parts_LoopBeginInc)p1).setValue(var.toString(), val1.toString(), val2.toString(), val3.toString());
			else
				((Parts_LoopBeginDec)p1).setValue(var.toString(), val1.toString(), val2.toString(), val3.toString());
			convertPen2Flowchart(node.jjtGetChild(4), b1, null);
			return b2;

		}
	}

	static private Parts makeRepeatuntil(ASTRepeatUntil node, Parts parts) throws ParseException {
		Parts_LoopBegin2 p1 = new Parts_LoopBegin2();
		Parts_LoopEnd2 p2 = new Parts_LoopEnd2();
		p1.setLoopEnd(p2);
		p2.setLoopBegin(p1);
		Parts_Bar b1 = new Parts_Bar();
		Parts_Bar b2 = new Parts_Bar();
		parts.connectLeft(p1);
		p1.connectLeft(b1);
		b1.connectLeft(p2);
		p2.connectLeft(b2);
		StringBuilder cond = new StringBuilder();
		setValue(node.jjtGetChild(1), cond);
		p2.setValue(cond.toString());
		convertPen2Flowchart(node.jjtGetChild(0), b1, null);
		return b2;
	}

	private static Parts makeMisc(String command, Node[] children, Parts parts) throws ParseException {
		Parts_Misc p1 = new Parts_Misc();
		String[] argv = null;
		int argc = children == null ? 0 : children.length;
		if(argc > 0)
		{
			argv = new String[argc];
			for(int i = 0; i < argc; i++)
			{
				StringBuilder sb = new StringBuilder();
				setValue(children[i], sb);
				argv[i] = sb.toString();
			}
		}
		p1.setValue(command, argv);
		parts.connectLeft(p1);
		parts = new Parts_Bar();
		p1.connectLeft(parts);
		return parts;
	}

	private static Parts makeFunctionCall(ASTFunctionCall child, Parts parts) throws ParseException {
		Parts_Misc p1 = new Parts_Misc();
		StringBuilder sb = new StringBuilder();
		sb.append(child.varName);
		sb.append('(');
		for(int i = 0; i < child.children.length; i++)
		{
			if(i > 0) sb.append(',');
			setValue(child.children[i], sb);
		}
		sb.append(')');
		p1.setValue("anyCommand",new String[]{sb.toString()});
		parts.connectLeft(p1);
		parts = new Parts_Bar();
		p1.connectLeft(parts);
		return parts;
	}

	private static Parts makeMiscSubstitute(String command, String var, SimpleNode node) throws ParseException {
		Parts_Misc p1 = new Parts_Misc();
		String[] argv = null;
		int argc = node.jjtGetNumChildren();
		argv = new String[argc + 1];
		argv[0] = var;
		if(argc > 0)
		{
			for(int i = 0; i < argc; i++)
			{
				StringBuilder sb = new StringBuilder();
				setValue(node.jjtGetChild(i), sb);
				argv[i + 1] = sb.toString();
			}
		}
		p1.setValue(command, argv);
		return p1;
	}

}
